@@ -1,7 +1,7 @@
 use crate::{Bundler, Load, Resolve};
 use swc_common::pass::Repeat;
 use swc_ecma_ast::*;
-use swc_ecma_transforms::optimization::simplify::{const_propgation::constant_propagation, dce};
+use swc_ecma_transforms_optimization::simplify::{const_propgation::constant_propagation, dce};
 use swc_ecma_visit::FoldWith;
 
 impl<L, R> Bundler<'_, L, R>
